@@ -101,7 +101,7 @@ impl FrequencySketch {
         }
 
         self.table = vec![0; table_size as usize].into_boxed_slice();
-        self.table_mask = 0.max(table_size - 1) as u64;
+        self.table_mask = table_size.saturating_sub(1) as u64;
         self.sample_size = if cap == 0 {
             10
         } else {
@@ -117,10 +117,11 @@ impl FrequencySketch {
         }
 
         let start = ((hash & 3) << 2) as u8;
-        let mut frequency = std::u8::MAX;
+        let mut frequency = u8::MAX;
         for i in 0..4 {
             let index = self.index_of(hash, i);
-            let count = (self.table[index] >> ((start + i) << 2) & 0xF) as u8;
+            let shift = (start + i) << 2;
+            let count = ((self.table[index] >> shift) & 0xF) as u8;
             frequency = frequency.min(count);
         }
         frequency
@@ -180,7 +181,7 @@ impl FrequencySketch {
     fn index_of(&self, hash: u64, depth: u8) -> usize {
         let i = depth as usize;
         let mut hash = hash.wrapping_add(SEED[i]).wrapping_mul(SEED[i]);
-        hash += hash >> 32;
+        hash = hash.wrapping_add(hash >> 32);
         (hash & self.table_mask) as usize
     }
 
@@ -206,12 +207,12 @@ impl FrequencySketch {
 mod tests {
     use super::FrequencySketch;
     use once_cell::sync::Lazy;
-    use std::hash::{BuildHasher, Hash, Hasher};
+    use std::hash::{BuildHasher, Hash};
 
     static ITEM: Lazy<u32> = Lazy::new(|| {
         let mut buf = [0; 4];
         getrandom::getrandom(&mut buf).unwrap();
-        unsafe { std::mem::transmute::<[u8; 4], u32>(buf) }
+        u32::from_ne_bytes(buf)
     });
 
     // This test was ported from Caffeine.
@@ -257,7 +258,7 @@ mod tests {
         let mut sketch = FrequencySketch::default();
         sketch.ensure_capacity(512);
         let mut indexes = std::collections::HashSet::new();
-        let hashes = vec![std::u64::MAX, 0, 1];
+        let hashes = [u64::MAX, 0, 1];
         for hash in hashes.iter() {
             for depth in 0..4 {
                 indexes.insert(sketch.index_of(*hash, depth));
@@ -321,10 +322,76 @@ mod tests {
 
     fn hasher<K: Hash>() -> impl Fn(K) -> u64 {
         let build_hasher = std::collections::hash_map::RandomState::default();
-        move |key| {
-            let mut hasher = build_hasher.build_hasher();
-            key.hash(&mut hasher);
-            hasher.finish()
+        move |key| build_hasher.hash_one(&key)
+    }
+}
+
+// Verify that some properties hold such as no panic occurs on any possible inputs.
+#[cfg(kani)]
+mod kani {
+    use super::FrequencySketch;
+
+    const CAPACITIES: &[u32] = &[
+        0,
+        1,
+        1024,
+        1025,
+        2u32.pow(24),
+        2u32.pow(24) + 1,
+        2u32.pow(30),
+        2u32.pow(30) + 1,
+        u32::MAX,
+    ];
+
+    #[kani::proof]
+    fn verify_ensure_capacity() {
+        // Check for arbitrary capacities.
+        let capacity = kani::any();
+        let mut sketch = FrequencySketch::default();
+        sketch.ensure_capacity(capacity);
+    }
+
+    #[kani::proof]
+    fn verify_frequency() {
+        // Check for some selected capacities.
+        for capacity in CAPACITIES {
+            let mut sketch = FrequencySketch::default();
+            sketch.ensure_capacity(*capacity);
+
+            // Check for arbitrary hashes.
+            let hash = kani::any();
+            let frequency = sketch.frequency(hash);
+            assert!(frequency <= 15);
+        }
+    }
+
+    #[kani::proof]
+    fn verify_increment() {
+        // Only check for small capacities. Because Kani Rust Verifier is a model
+        // checking tool, it will take much longer time (exponential) to check larger
+        // capacities here.
+        for capacity in &[0, 1, 128] {
+            let mut sketch = FrequencySketch::default();
+            sketch.ensure_capacity(*capacity);
+
+            // Check for arbitrary hashes.
+            let hash = kani::any();
+            sketch.increment(hash);
+        }
+    }
+
+    #[kani::proof]
+    fn verify_index_of() {
+        // Check for arbitrary capacities.
+        let capacity = kani::any();
+        let mut sketch = FrequencySketch::default();
+        sketch.ensure_capacity(capacity);
+
+        // Check for arbitrary hashes.
+        let hash = kani::any();
+        for i in 0..4 {
+            let index = sketch.index_of(hash, i);
+            assert!(index < sketch.table.len());
         }
     }
 }
